@@ -1,21 +1,27 @@
 from collections import defaultdict
-from typing import Dict, List
+from typing import Dict, List, Any
 
 from flask import Flask
 
 from . import utils
 
 
-def generate_spec(openapi_version, title, version, app: Flask, models: Dict) -> Dict:
+def generate_openapi(title: str,
+                     version: str,
+                     openapi_version: str,
+                     app: Flask,
+                     models: Dict[str, Dict],
+                     description: str = None) -> Dict[str, Any]:
     """
-    生成openapi json
-    :param openapi_version:  openapi版本号
-    :param title:   文档标题
-    :param version:  文档版本
-    :param app:  flask app
-    :param models:  Pydantic model 字典集合
+    :param title:
+    :param version:
+    :param openapi_version:
+    :param app:
+    :param models:
+    :param description:
     :return:
     """
+
     routes: Dict[str:Dict] = dict()
     tags: Dict[str:Dict] = dict()
     groups: Dict[str:List] = defaultdict(list)
@@ -23,7 +29,7 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
         # 视图函数
         func = app.view_functions[rule.endpoint]
 
-        path, parameters = utils.parse_route_to_path_params(str(rule))
+        path, parameters = utils.parse_path_params(str(rule))
         # 只有被siwadoc装饰了函数才加入openapi
         if not getattr(func, '_decorated', None):
             continue
@@ -42,17 +48,15 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
 
             groups[func_group].extend(func_tags)
             tags.update({tag: {"name": tag} for tag in func_tags})
-
-            summary, description = utils.get_func_doc(func)
-            spec = {
-                'summary': summary or func.__name__.capitalize(),
-                'description': description or '',
+            operation = {
+                'summary': utils.get_operation_summary(func),
+                'description': utils.get_operation_description(func),
                 'operationID': func.__name__ + '__' + method.lower(),
                 'tags': func_tags,
             }
 
             if hasattr(func, 'body'):
-                spec['requestBody'] = {
+                operation['requestBody'] = {
                     'content': {
                         'application/json': {
                             'schema': {
@@ -61,30 +65,26 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
                         }
                     }
                 }
-
             if hasattr(func, 'query'):
-                parameters.append({
-                    'name': func.query,
-                    'in': 'query',
-                    'required': False,
-                    'schema': {
-                        '$ref': f'#/components/schemas/{func.query}',
-                    }
-                })
-            spec['parameters'] = parameters
+                parameters.extend(utils.parse_other_params('query', models[func.query]))
+            if hasattr(func, 'header'):
+                parameters.extend(utils.parse_other_params('header', models[func.header]))
+            if hasattr(func, 'cookie'):
+                parameters.extend(utils.parse_other_params('cookie', models[func.cookie]))
+            operation['parameters'] = parameters
 
-            spec['responses'] = {}
+            operation['responses'] = {}
             has_2xx = False
             if hasattr(func, 'x'):
                 for code, msg in func.x.items():
                     if code.startswith('2'):
                         has_2xx = True
-                    spec['responses'][code] = {
+                    operation['responses'][code] = {
                         'description': msg,
                     }
 
             if hasattr(func, 'resp'):
-                spec['responses']['200'] = {
+                operation['responses']['200'] = {
                     'description': 'Successful Response',
                     'content': {
                         'application/json': {
@@ -95,10 +95,10 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
                     },
                 }
             elif not has_2xx:
-                spec['responses']['200'] = {'description': 'Successful Response'}
+                operation['responses']['200'] = {'description': 'Successful Response'}
 
             if any([hasattr(func, schema) for schema in ('query', 'body')]):
-                spec['responses']['400'] = {
+                operation['responses']['400'] = {
                     'description': 'Validation Error',
                     'content': {
                         'application/json': {
@@ -108,7 +108,7 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
                         }
                     },
                 }
-            routes.setdefault(path, {})[method.lower()] = spec
+            routes.setdefault(path, {})[method.lower()] = operation
 
     definitions = {}
     for _, schema in models.items():
@@ -116,13 +116,15 @@ def generate_spec(openapi_version, title, version, app: Flask, models: Dict) ->
             for key, value in schema['definitions'].items():
                 definitions[key] = value
             del schema['definitions']
-
+    info = {
+        'title': title,
+        'version': version,
+    }
+    if description:
+        info["description"] = description
     data = {
         'openapi': openapi_version,
-        'info': {
-            'title': title,
-            'version': version,
-        },
+        'info': info,
         'tags': list(tags.values()),
         'x-tagGroups': [{"name": k, "tags": list(set(v))} for k, v in groups.items()],
         'paths': {
